
      SUBROUTINE USR_RATES_DES(NP, pM, IJK, DES_RATES)

            USE compar
            USE constant
            USE des_thermo       !des_T_s, des_C_ps
            USE des_rxns
            USE discretelement   !pmass-particle mass
            USE energy
            USE fldvar
            USE funits
            USE geometry
            USE indices
            USE param
            USE param1
            USE physprop
            USE rxns
            USE run
            USE usr
            USE fun_avg
            USE functions
      
            USE parallel
            USE sendrecv
            USE toleranc
      
      
      
            IMPLICIT NONE
      
            INTEGER, INTENT(IN) :: NP  ! Global index of particle
            INTEGER, INTENT(IN) :: PM  ! Solid phase index of particle NP
            INTEGER, INTENT(IN) :: IJK ! Fluid cell index containing NP
      
      ! Calculated reaction rates. (reacted moles per sec)
            DOUBLE PRECISION, INTENT(OUT) :: DES_RATES(NO_OF_DES_RXNS)
      
      
      ! Reaction specific variables:
      !`````````````````````````````````````````````````````````````````````//
      ! Solids Diameter (m)
            DOUBLE PRECISION :: Dp0

      ! Gas phase concentrations (mol/m^3)
            DOUBLE PRECISION c_CO,  c_H2, c_H2O, c_CO2
      ! Partial pressures (KPa)
            DOUBLE PRECISION p_CO,  p_H2, p_H2O, p_CO2

      ! Bounded phase temperatures (K)
            DOUBLE PRECISION xTg   ! gas phase
            DOUBLE PRECISION xTs   ! solids phase
            
            DOUBLE PRECISION E_i, A_i, C_i, A_p
            DOUBLE PRECISION r_diff, r_kin
            integer LB,UB
            
      
      ! Prevent reactions when mass fraction falls below c_Limiter.
            DOUBLE PRECISION, parameter :: c_Limiter = 1.0d-7
      
      !     Uncomment the following line when customizing this usr_rates_des.f to include generated file species.inc:
      !
           INCLUDE 'species.inc'
      
      
      ! Reaction rates:
      !`````````````````````````````````````````````````````````````````````//
      
            IF (PM==2) then !Only the biomass would react, sand(PM==1) will not.
            IF(pmass(NP)>1.0d-10) then !prevent too small particles
      
      ! Calculate particle diameter.
            Dp0 = 2.0d0 * DES_RADIUS(NP)
            !write(*,*) ''
            write(*,*) '****************************'
            write(*,*) 'A usr_rates_des is called'
            write(*,*) 'NP= ', NP
            write(*,*) 'PM ', PM
            write(*,*) 'Dp0= ', Dp0
      
      ! Calculate the bounded temperatures.
            xTg = max(min(T_g(IJK), 2500.0), 300.0d0)
            xTs = max(min(des_T_s(NP), 2500.0), 300.0d0)
            
      ! Particle surface area (m^2)
            A_p = Pi* Dp0 * Dp0   ! (m^2)
            !write(*,*) 'Ap= ', A_p
      ! Initialize partial pressures and concentrations.
            c_CO   = ZERO;     p_CO = ZERO;
            c_H2O  = ZERO;     p_H2O = ZERO;
            c_H2   = ZERO;     p_H2   = ZERO;
            c_CO2   = ZERO;    p_CO2   = ZERO;
      
      ! Calculate the partial pressure and molar concentration of H2O
            IF(X_g(IJK,H2O) > c_Limiter) then
                  p_H2O = 1.0d-3*P_g(IJK) * X_g(IJK,H2O) * (MW_MIX_g(IJK) / MW_g(H2O))
                  c_H2O = RO_g(IJK) * X_g(IJK,H2O) / Mw_g(H2O)
            ENDIF
                  !write(*,*) 'p_H2O= ', p_H2O
                  !write(*,*) 'c_H2O= ', c_H2O
                  
      ! Calculate the partial pressure and molar concentration of CO2
            IF(X_g(IJK,CO2) > c_Limiter) then
               p_CO2 = 1.0d-3*P_g(IJK) * X_g(IJK,CO2) * (MW_MIX_g(IJK) / MW_g(CO2))
               c_CO2 = RO_g(IJK) * X_g(IJK,CO2) / Mw_g(CO2)
            ENDIF
               !write(*,*) 'p_CO2= ', p_CO2
               !write(*,*) 'c_CO2= ', c_CO2
       
       
      !**********************************************************************!
      !                         Pyrolysis                                    !
      !**********************************************************************!     
      
      if(EP_s(IJK,2) .NE. 0.0) then
      !only react when there is biomass particle
      
      ! H2O(liquid) --> H2O(gas)  (m^3/kmole*s)
      !---------------------------------------------------------------------//
          if(des_X_s(NP,moisture) > c_Limiter) then
            DES_RATES(Drying) = 5.13d6 * des_X_s(NP, moisture) * EXP(-8.79d4/8.314/xTs)
          else
            DES_RATES(Drying) = ZERO
          endif
       write(*,*) 'DES_RATES(Drying)=', DES_RATES(Drying)
      
      ! wood --> 0.978CO + 0.689CO2 + 0.5363H2 + 0.7954CH4 + 1.333char  (m^3/kmole*s)
      !---------------------------------------------------------------------//
          if(des_X_s(NP,wood) > c_Limiter) then
            DES_RATES(Pyrolysis) = 5.0d6 * des_X_s(NP, wood) * EXP(-1.2d5/8.314/xTs)
          else
            DES_RATES(Pyrolysis) = ZERO
          endif
      write(*,*) 'des_X_s(NP,wood)=', des_X_s(NP,wood)  
      write(*,*) 'DES_RATES(Pyrolysis)=', DES_RATES(Pyrolysis)
      
            
      !**********************************************************************!
      !                        Heterogeneous Reactions                       !
      !**********************************************************************!
      ! char + CO2 --> 2CO  (m^3/kmole*s)
      !---------------------------------------------------------------------//
         if(des_X_s(NP,wood) > c_Limiter) then
                  C_i=5d-12
                  A_i=8.3
                  E_i=4.37d4
               r_kin   = A_i * xTs * exp(-E_i/(8.314 * xTs))
               r_diff  = C_i * ((xTs + xTg)/2)**0.75/Dp0
               DES_RATES(R1)  = 1.0/MW_s(2,char) *A_p*p_CO2*(r_diff*r_kin)/(r_diff+r_kin)
         else
               DES_RATES(R1) = ZERO
         endif 
       write(*,*) 'DES_RATES(R1)=', DES_RATES(R1)
       
      
      ! char + H2O --> CO + H2  (m^3/kmole*s)
      !---------------------------------------------------------------------//
         if(des_X_s(NP,wood) > c_Limiter) then
                  C_i=5d-12
                  A_i=45.6
                  E_i=4.37d4
               r_kin   = A_i * xTs * exp(-E_i/(8.314 * xTs))
               r_diff  = C_i * ((xTs + xTg)/2)**0.75/Dp0
               DES_RATES(R2)  = 1.0/MW_s(2,char) * A_p*p_H2O*(r_diff*r_kin)/(r_diff+r_kin)
         else
               DES_RATES(R2) = ZERO
         endif
      write(*,*) 'DES_RATES(R2)=', DES_RATES(R2) 
      write(*,*) 'pmass(NP)=', pmass(NP)
      
      endif   !end if of .not. compare(EP_s(IJK,2),ZERO)
      
            !IF(nrr>=Rs1) ReactionRates(IJK,Rs1) = DES_RATES(Rs1)
            !IF(nrr>=Rs_CO2) ReactionRates(IJK,RS_CO2) = DES_RATES(RS_CO2)
            !IF(nrr>=Rs_H2O) ReactionRates(IJK,RS_H2O) = DES_RATES(RS_H2O)
               LB= NO_OF_RXNS+1
               UB= NO_OF_RXNS+NO_OF_DES_RXNS
               !ReactionRates(IJK,LB:UB) = ReactionRates(IJK,LB:UB) + DES_RATES
               !ReactionRates(IJK,LB:UB) =  DES_RATES
               ReactionRates(IJK,LB) =  ReactionRates(IJK,LB)     + DES_RATES(Drying)
               ReactionRates(IJK,LB+1) =  ReactionRates(IJK,LB+1) + DES_RATES(Pyrolysis)
               ReactionRates(IJK,LB+2) =  ReactionRates(IJK,LB+2) + DES_RATES(R1)
               ReactionRates(IJK,LB+3) =  ReactionRates(IJK,LB+3) + DES_RATES(R2)
               
            endif  !end if of pmass>1.0d-10   
            endif  !end if of PM==2   
               
            RETURN
      
            END SUBROUTINE USR_RATES_DES
      